########################################################################################################################
# Constants and analysis settings
########################################################################################################################
EXPERIMENT_DATA_DIR = paste(dirname(sys.frame(-3)$ofile), '/../raw-data/', sep='') # This works in RStudio
GENERATED_IMAGES_DIR = paste(EXPERIMENT_DATA_DIR, "../analysis/", sep='')
TOTAL_NUM_OBSERVATIONS = 675

SAMPLES_PER_QUESTION = 15
LEADING_SAMPLES_TO_EXCLUDE = 3
AVERAGE_WITHIN_SUBJECT_SAMPLES = TRUE

QUESTION_LEVELS = c("throw in body?", "all variables immediately initialized?", "uses subtraction?", "uses 'this'?", "string local variable?",
                    "if with else?" , "int second parameter?", "nested loops?", "3 parameters?",  "top-level loop after any if?",
                    "creates new objects?", "catches any exceptions?", "loop with 2+ method/constructor calls?", "multiple returns?", "uses type cast?")

CONDITION_LEVELS = c("eclipse", "envision reduced", "envision default")

ECLIPSE_COLOR="gray90"
REDUCED_COLOR="#e6b8af"
ENVISION_COLOR="#6d9eeb"

########################################################################################################################
# Imports
########################################################################################################################
#install.packages("compute.es"); install.packages("car"); install.packages("ggplot2"); install.packages("multcomp");
#install.packages("pastecs"); install.packages("ez"); install.packages("readODS"); install.packages("HH");
#install.packages("likert"); install.packages("vioplot"); install.packages("lme4"); install.packages("nlme");
#install.packages("doBy"); install.packages("pgirmess"); install.packages("PairedData")

#In Ubuntu 15.10, the following are needed to install all packages:
#sudo apt-get install libmpfr4 libmpfr-dev libcurl4-gnutls-dev libgdal1-dev libproj-dev

library(compute.es)
library(car)
library(ggplot2)
library(multcomp)
library(pastecs)
library(ez)
library(readODS)
library(HH)
library(lme4)
library(nlme)
library(doBy)
library(MASS)
library(pgirmess)
library(PairedData)

###    This is needed for robust methods by WRS: source http://www.nicebread.de/installation-of-wrs-package-wilcox-robust-statistics/
###    first: install dependent packages
#install.packages(c("MASS", "akima", "robustbase"))
###    second: install suggested packages
#install.packages(c("cobs", "robust", "mgcv", "scatterplot3d", "quantreg", "rrcov", "lars", "pwr", "trimcluster", "parallel", "mc2d", "psych", "Rfit"))
###    third: install an additional package which provides some C functions
#install.packages("devtools")
#library(devtools)
#install_github("mrxiaohe/WRScpp")
###    fourth: install WRS
#install_github("nicebread/WRS", subdir="pkg")

library(WRS)


########################################################################################################################
# Read the correct answer sheet
########################################################################################################################
answers = read.ods(paste(EXPERIMENT_DATA_DIR, "../analysis/answers.ods", sep=''))[[1]]
colnames(answers) = answers[1, ]
answers = answers[-1, ]

isAnswerCorrect = function(sampleFileName, questionHint, givenAnswer) {
  stopifnot(givenAnswer == "n" || givenAnswer == "return" || givenAnswer == "enter")
  
  index = regexpr("\\d\\d\\d\\.png$", sampleFileName)
  stopifnot(index > 0)
  
  sampleNumber = strtoi(substr(sampleFileName, index, index+2), 10)
  stopifnot(index > 0)
  
  correctAnswer = answers[sampleNumber, questionHint]
  stopifnot(correctAnswer == "no" || correctAnswer == "yes")
  
  return ( (correctAnswer == "no" && givenAnswer == "n") || (correctAnswer == "yes" && givenAnswer != "n") )
}

encodeCorrectAnswer = function(dataFrameRow) {
  correct = isAnswerCorrect(dataFrameRow[3], dataFrameRow[1], dataFrameRow[5])
  return (if (correct) 1 else 0)
}

########################################################################################################################
# Participant data reading functions
########################################################################################################################
dataOneParticipant = function(fileName, averageSamples, onlyUseAnswers) {
  df = read.csv( paste(EXPERIMENT_DATA_DIR, fileName, sep=''))
  
  qualitative = readQualitative(df)
  
  # Cut the qualitative answers at the end
  df = df[1:TOTAL_NUM_OBSERVATIONS,]
  df = droplevels(df)
  df$Condition = factor(df$Condition, levels = CONDITION_LEVELS)
  df$Question = factor(df$Question, levels = QUESTION_LEVELS)
  
  # Ignore the initial few observations to exclude the initial learning curve
  if (LEADING_SAMPLES_TO_EXCLUDE > 0)
  {
    for (i in 1:LEADING_SAMPLES_TO_EXCLUDE)
    {
      df = df[ -(seq(1, to=nrow(df), by=SAMPLES_PER_QUESTION - (i-1))),]
    }
  }
  
  # Filter by answer
  if (!missing(onlyUseAnswers))
  {
    stopifnot(onlyUseAnswers == "no" || onlyUserAnswers == "yes")
    
    if (onlyUserAnswers == "no")
      df = df[ which(df$Response=="n"), ]
    else
      df = df[ which(df$Response!="n"), ]
  }
  
  # Add a column that represents whether an answer was correct or not
  df$Correct = apply(df, 1, encodeCorrectAnswer)
    
  # Skip the file names and response answer
  df = df[c(1,2,4,6)]
  df = transform(df, Time = as.numeric(as.character(Time)))
  
  # Convert time from ms to s
  df$Time = df$Time / 1000
  
  if (averageSamples == TRUE)
  {
    df = aggregate(df[,3:4], list(df$Question, df$Condition), mean)
    colnames(df) = c("Question", "Condition", "Time", "Correct")
  }
  
  # Add a column that represents the category of a question
  df$QCategory = lapply(df$Question, questionHintToCategory)
  df$QCategory = factor(df$QCategory, levels = c(1,2,3))
  
  # Add columns representing the qualitative answers
  replicated = qualitative[rep(seq_len(nrow(qualitative)), nrow(df)), ]
  
  # Add a column that marks whether Envision has been seen or not
  replicated$SeenOrUsedEnvision = lapply(replicated$EnvisionExp, usedOrSeenEnvision)
  replicated$SeenOrUsedEnvision = factor(replicated$SeenOrUsedEnvision, levels = c(0,1))
  
  # Add a marker for the current participant
  ParticipantNumber = substr(fileName, 9, nchar(fileName)-4)
  Participant = rep(ParticipantNumber, nrow(df))
  qualitative = data.frame(Participant = ParticipantNumber, qualitative)
  
  return (list(Data = data.frame(Participant, df, replicated), Qualitative = qualitative))
}

questionHintToCategory = function(hint) {
  questionIndex = which(QUESTION_LEVELS == hint)[1]
  stopifnot(questionIndex >=1 && questionIndex <=15)
  return ((questionIndex-1) %/% 5 + 1)
}

usedOrSeenEnvision = function(x) {
  if (x == 0) return (0) else return (1)
}

readQualitative = function(rawData) {
  return (data.frame(Icons = textToNumber(rawData[[678,1]])-2, #rememberIcons
                     Colors = textToNumber(rawData[[678,2]])-2, #rememberColors
                     Understand = textToNumber(rawData[[678,3]])-2, #understandVisualizations
                     EclipseRating = textToNumber(rawData[[678,4]])-1, #eclipseFavorite
                     ReducedRating = textToNumber(rawData[[678,5]])-1, #envisionReducedFavorite
                     DefaultRating = textToNumber(rawData[[679,1]])-1, #envisionDefaultFavorite
                     Age = textToNumber(rawData[[683,1]]), #age
                     Gender = textToNumber(rawData[[683,2]]), #gender
                     Degree = textToNumber(rawData[[683,3]]), #degree
                     JavaExp = textToNumber(rawData[[683,4]]), #javaExperience
                     ProgrammingExp = textToNumber(rawData[[683,5]]), #programmingExperience
                     ProfessionalExp = textToNumber(rawData[[684,1]]), #professionalExperience
                     EclipseExp = textToNumber(rawData[[684,2]]), #eclipseExperience
                     Hours = textToNumber(rawData[[684,3]]), #hoursProgramming
                     EnvisionExp = textToNumber(rawData[[684,4]]), #usedEnvision
                     Colorblind = textToNumber(rawData[[685,1]]), #colorBlind
                     Editors = rawData[[684,5]], #usedEditors
                     Comment = rawData[[679,2]]  #wouldUseEnvision
            ))
}

textToNumber = function(text) {
  # Java experience text answers
  if (text == "I know Java for roughly 10 years, but never programmed something serious in it.") number = 10
  else if (text == "9 years with switching to other languages in between") number = 9
  else if (text == "more than 5") number = 5
  else if (text == "9 but I do not use it actively") number = 9
  else if (text == "1.5-2") number = 1.75
  
  # Programming experience text answers
  else if (text == "Roughly 20.") number = 20
  else if (text == "9 years") number = 9
  else if (text == "Around 12 years") number = 12
  else if (text == "3-4..") number = 3.5
  
  # Professional experience text answers
  else if (text == "Around 3 years in total.") number = 3
  else if (text == "1.5 years") number = 1.5
  else if (text == "more than 5 (research software though)") number = 5
  else if (text == "about 6 months") number = 0.5
  else if (text == "none") number = 0
  
  # Eclipse experience text answers
  else if (text == "1 month") number = 0.083
  else if (text == "8 years") number = 8
  else if (text == "Occasionally, more than 5") number = 5
  else if (text == "4-5") number = 4.5
  
  # Standard number answers
  else number = as.numeric(as.character(text))
  
  return (number)
}

dataAllParticipants = function(dataDir) {
  dataFileNames = list.files(dataDir)
  
  allData = data.frame(Participant=character(),Question=character(),Condition=character(),Time=numeric(), stringsAsFactors=FALSE)
  allQualitative = data.frame(Icons=numeric(), Colors=numeric(), Understand=numeric(),
                              EclipseRating=numeric(), ReducedRating=numeric(), DefaultRating=numeric(),
                              Age=numeric(), Gender=numeric(), Degree=numeric(), Java=numeric(), Programming=numeric(),
                              Professional=numeric(), Eclipse=numeric(), Hours=numeric(), Familiar=numeric(), Colorblind=numeric(),
                              Editors=character(), Comment=character(), stringsAsFactors=FALSE)
  
  for (fileName in dataFileNames) {
    one = dataOneParticipant(fileName, AVERAGE_WITHIN_SUBJECT_SAMPLES)
    allData = rbind(allData, one$Data);
    allQualitative = rbind(allQualitative, one$Qualitative)
  }
  
  return (list(Data = allData, Qualitative = allQualitative))
}

########################################################################################################################
# Read all data
########################################################################################################################
all = dataAllParticipants(EXPERIMENT_DATA_DIR)
allData = all$Data
allQualitative = all$Qualitative

########################################################################################################################
# Summary box plots
########################################################################################################################
BOX_PLOT_NAMES = c( "Q1", "Q2", "Q3", "Q4", "Q5", "Q6", "Q7", "Q8", "Q9", "Q10", "Q11", "Q12", "Q13", "Q14", "Q15")

pdf(paste(GENERATED_IMAGES_DIR, "boxplot-time.pdf", sep=''), width=12, height=4.5)
boxplot(Time~Condition+Question , data=allData, col=c(ECLIPSE_COLOR,REDUCED_COLOR,ENVISION_COLOR), las = 2, par(mar = c(1, 5, 0.5, 2)),
        at =c(1:3, 5:7, 9:11, 13:15, 17:19, 23:25, 27:29, 31:33, 35:37, 39:41, 45:47, 49:51, 53:55, 57:59, 61:63),
        xaxt='n', ylab="response time [s]", ylim=c(0,16.2))
legend(28.5, 15, c("v-low", "v-med", "v-high"), fill = c(ECLIPSE_COLOR,REDUCED_COLOR,ENVISION_COLOR))
abline(v = 21, col = "gray60")
abline(v = 43, col = "gray60")
text(c(2,6,10,14,18,24,28,32,36,40,46,50,54,58,62)+1, par("usr")[3] - 0.5, adj = 1, xpd = TRUE, labels=BOX_PLOT_NAMES)
text(x = 62, y = 16.85, labels = "X", xpd = NA, col="red")
dev.off()

pdf(paste(GENERATED_IMAGES_DIR, "boxplot-correctness.pdf", sep=''), width=12, height=1.5)
boxplot(Correct*100~Condition+Question , data=allData, col=c(ECLIPSE_COLOR,REDUCED_COLOR,ENVISION_COLOR), las = 2, par(mar = c(0.2, 5, 1, 2)),
        at =c(1:3, 5:7, 9:11, 13:15, 17:19, 23:25, 27:29, 31:33, 35:37, 39:41, 45:47, 49:51, 53:55, 57:59, 61:63),
        xaxt='n', ylab="% correct")
abline(v = 21, col = "gray60")
abline(v = 43, col = "gray60")
mtext(c("unenhanced","enhanced by both v-mid and v-high","enhanced mainly by v-high"), at=c(10, 32, 54))
dev.off()

########################################################################################################################
# Likert questions
########################################################################################################################
#par(mar = c(5, 4, 4, 2))
#hist(allQualitative$Icons, breaks=c(-2.5, -1.5, -0.5, 0.5, 1.5, 2.5),
#     main="How easy was it to remember Envision's icons and symbols?")
#hist(allQualitative$Colors, breaks=c(-2.5, -1.5, -0.5, 0.5, 1.5, 2.5),
#     main="How easy was it to remember Envision's colors?")
#hist(allQualitative$Understand, breaks=c(-2.5, -1.5, -0.5, 0.5, 1.5, 2.5),
#     main="Overall, how easy was it to understand Envision's\nvisualizations compared to Eclipse?")

likertQuestions = data.frame (
  "very_difficult"  = c(sum(allQualitative$Icons == -2),sum(allQualitative$Colors == -2),sum(allQualitative$Understand == -2)),
  "difficult"       = c(sum(allQualitative$Icons == -1),sum(allQualitative$Colors == -1),sum(allQualitative$Understand == -1)),
  "the_same"        = c(sum(allQualitative$Icons == 0),sum(allQualitative$Colors == 0),sum(allQualitative$Understand == -0)),
  "easy"            = c(sum(allQualitative$Icons == 1),sum(allQualitative$Colors == 1),sum(allQualitative$Understand == 1)),
  "very_easy"       = c(sum(allQualitative$Icons == 2),sum(allQualitative$Colors == 2),sum(allQualitative$Understand == 2))
)
colnames(likertQuestions) <- c("-2 (very difficult)", "-1", "0", "1", "2 (very easy)")
rownames(likertQuestions) = c("icons",
                              "colors",
                              "Eclipse")

pdf(paste(GENERATED_IMAGES_DIR, "likert.pdf", sep=''), width=4.2, height=3)
LikertPlot = likert(likertQuestions, horizontal = FALSE,
                    main = NULL,
                    xlab = list(label="# responses"),
                    auto.key = list(space = "bottom"),
                    par.settings=list(layout.heights=list(key.axis.padding=0,  top.padding=0, bottom.padding=0, axis.bottom=0),
                                      layout.widths=list(left.padding=0, right.padding=0)
                    ))
print(LikertPlot)
dev.off()

########################################################################################################################
# Tool ranking
########################################################################################################################

rankings = rbind(data.frame(Tool="Eclipse", Rank=allQualitative$EclipseRating),
            data.frame(Tool="Envision red.", Rank=allQualitative$ReducedRating),
            data.frame(Tool="Envision def.", Rank=allQualitative$DefaultRating))
rankings$Tool = as.factor(rankings$Tool)
rankings$Rank = factor(rankings$Rank, levels = c(-1,0,1))

pdf(paste(GENERATED_IMAGES_DIR, "ranking.pdf", sep=''), width=3.3, height=2.2)
rankPlot = ggplot(rankings, aes(x=Rank, fill=Tool)) +
  geom_bar(color="black", position="dodge", width=0.7) +
  scale_fill_manual("", values=c(ECLIPSE_COLOR,REDUCED_COLOR,ENVISION_COLOR)) +
  theme_classic() +
  theme(text = element_text(size=9)) +
  theme(legend.position=c(0.4, 1.1), legend.direction="horizontal") + 
  guides(fill = guide_legend(override.aes = list(colour = NULL))) + 
  theme(legend.key = element_rect(colour = "black")) +
  ggtitle(" ") +
  theme(axis.title.x=element_text(vjust=0)) +
  theme(axis.title.y=element_text(vjust=1.25)) +
  xlab("Tool rank on method rendering")
print(rankPlot)
dev.off()

# Textual summary:
summaryBy(Rank+1 ~ Tool, data = rankings, FUN = function(x) { c(m = mean(x), s = sd(x)) } )

########################################################################################################################
# Pairwise T-Tests
########################################################################################################################

shortConditions = function(condition) {
  if (condition == "eclipse") return ("e")
  if (condition == "envision reduced") return ("r")
  if (condition == "envision default") return ("d")
  
  stop("unrecognized condition");
}

shortQuestions = function(question) {
  m = (match(question, QUESTION_LEVELS));
  if (nchar(m) == 1) m = paste("0", m, sep = "")
  return (m)
}

groups = paste(lapply(allData$Question, shortQuestions), lapply(allData$Condition, shortConditions), sep="_")
time_pairwise = pairwise.t.test(allData$Time, groups, paired = TRUE, p.adjust.method = "bonferroni")
correct_pairwise = pairwise.t.test(allData$Correct, groups, paired = TRUE, p.adjust.method = "bonferroni")

groupsQCat = paste(allData$QCategory, lapply(allData$Condition, shortConditions), sep="_")
time_qcat_pairwise = pairwise.t.test(allData$Time, groupsQCat, paired = TRUE, p.adjust.method = "bonferroni")

interesting_simple_comparisons = function() {
  unadjusted = c()
  for (i in 1:length(QUESTION_LEVELS)) {
    low_mid = yuen.t.test(x = subset(allData, Question==QUESTION_LEVELS[i] & Condition == "eclipse", select = c(Time))[['Time']],
                          y = subset(allData, Question==QUESTION_LEVELS[i] & Condition == "envision reduced", select = c(Time))[['Time']],
                          paired = TRUE)
    
    low_high = yuen.t.test(x = subset(allData, Question==QUESTION_LEVELS[i] & Condition == "eclipse", select = c(Time))[['Time']],
                           y = subset(allData, Question==QUESTION_LEVELS[i] & Condition == "envision default", select = c(Time))[['Time']],
                           paired = TRUE)
    
    mid_high = yuen.t.test(x = subset(allData, Question==QUESTION_LEVELS[i] & Condition == "envision reduced", select = c(Time))[['Time']],
                           y = subset(allData, Question==QUESTION_LEVELS[i] & Condition == "envision default", select = c(Time))[['Time']],
                           paired = TRUE)
    
    unadjusted = c(unadjusted, low_mid$p.value, low_high$p.value, mid_high$p.value)
  }

  adjusted = p.adjust(unadjusted, method = "bonferroni")
  adjusted = matrix(adjusted, nrow = 15, ncol = 3, byrow = TRUE)
  colnames(adjusted) =  c("L-M", "L-H", "M-H")
  rownames(adjusted) =  paste("Q", c(1:15), sep="")
  
  return (adjusted)
}
interesting_simple_comparisons()
  
########################################################################################################################
# Robust ANOVAs
########################################################################################################################
robustCompareOneQuestion = function(question_text, wide) {
  trimmed_means = lloc(wide, tr=0.2)
  control_mean = trimmed_means[[1]]
  v_mid_mean = trimmed_means[[2]]
  v_high_mean = trimmed_means[[3]]
  
  robust_anova = rmanovab(wide, tr=.2, nboot=2000)
  pairwise_comparisons = pairdepb(wide, tr=.2, nboot=2000)
  
  data.frame(question=question_text, F_t = robust_anova$teststat, F_crit = robust_anova$crit, sig = robust_anova$teststat >= robust_anova$crit,
                   
             L_M_sig = pairwise_comparisons$crit < abs(pairwise_comparisons$test[[1,3]]) &&
               sign(pairwise_comparisons$psihat[[1,4]]) == sign(pairwise_comparisons$psihat[[1,5]]),
             L_M_psi = pairwise_comparisons$psihat[[1,3]], L_M_cil = pairwise_comparisons$psihat[[1,4]], L_M_cih = pairwise_comparisons$psihat[[1,5]],
             L_M_size = pairwise_comparisons$psihat[[1,3]] / control_mean * 100,
             
             L_H_sig = pairwise_comparisons$crit < abs(pairwise_comparisons$test[[2,3]]) &&
               sign(pairwise_comparisons$psihat[[2,4]]) == sign(pairwise_comparisons$psihat[[2,5]]),
             L_H_psi = pairwise_comparisons$psihat[[2,3]], L_H_cil = pairwise_comparisons$psihat[[2,4]], L_H_cih = pairwise_comparisons$psihat[[2,5]],
             L_H_size = pairwise_comparisons$psihat[[2,3]] / control_mean * 100,
             
             M_H_sig = pairwise_comparisons$crit < abs(pairwise_comparisons$test[[3,3]]) &&
               sign(pairwise_comparisons$psihat[[3,4]]) == sign(pairwise_comparisons$psihat[[3,5]]),
             M_H_psi = pairwise_comparisons$psihat[[3,3]], M_H_cil = pairwise_comparisons$psihat[[3,4]], M_H_cih = pairwise_comparisons$psihat[[3,5]],
             M_H_size = pairwise_comparisons$psihat[[3,3]] / control_mean * 100,
             
             control_mean = control_mean, v_mid_mean = v_mid_mean, v_high_mean = v_high_mean)
}

robustANOVAs = function(fun, data) {
  for (q in 1:length(QUESTION_LEVELS))
  {
    question_text = QUESTION_LEVELS[[q]]
    data_subset = subset(data, Question==question_text)[c(-1)]
    wide_format = reshape(data_subset, idvar = "Participant", timevar = "Condition", direction = "wide")[,-c(1)]
    analysis = fun(question_text, wide_format)
    
    if (q==1) res = analysis
    else res = rbind(res, analysis)
  }
  
  return (res)
}

# Uncomment these to run the robust ANOVA tests. They take a while.
#robustANOVAs(robustCompareOneQuestion, subset(allData, select=c(Question, Participant,  Condition, Time)))
#robustANOVAs(robustCompareOneQuestion, subset(allData, select=c(Question, Participant, Condition, Correct)))

robustConfidenceIntervalOneQuestion = function(question_text, wide)
{
  ci_vi_low = onesampb(wide[[1]], est=mean, nboot=2000, tr=0.2)
  ci_vi_med = onesampb(wide[[2]], est=mean, nboot=2000, tr=0.2)
  ci_vi_high = onesampb(wide[[3]], est=mean, nboot=2000, tr=0.2)
  
  tools = c("v-low", "v-med", "v-high")
  means = c(ci_vi_low[[3]], ci_vi_med[[3]], ci_vi_high[[3]])
  ci_left = c(ci_vi_low[[1]][[1]], ci_vi_med[[1]][[1]], ci_vi_high[[1]][[1]])
  ci_right = c(ci_vi_low[[1]][[2]], ci_vi_med[[1]][[2]], ci_vi_high[[1]][[2]])
  
  data = data.frame(question = rep(which(QUESTION_LEVELS == question_text)[1], 3), tool = tools, mean = means, ci_left = ci_left, ci_right = ci_right)
  
  return (data)
}
plotData = robustANOVAs(robustConfidenceIntervalOneQuestion, subset(allData, select=c(Question, Participant,  Condition, Time)))

plotRobustAnova = function(data)
{
  data$tool = factor(data$tool, levels = c("v-high", "v-med", "v-low")) # in order to get the correct order
  g_plot = ggplot(data, aes(x=question, y=mean, fill=tool)) + theme_classic() + theme(legend.position = c(0.8, 0.5)) +
    geom_bar(position=position_dodge(), stat="identity") + scale_fill_manual("",values=c(ENVISION_COLOR,REDUCED_COLOR,ECLIPSE_COLOR)) +
    theme(axis.line.y = element_blank(), axis.ticks.y = element_blank(), axis.title.y=element_blank()) +
    theme(plot.margin = unit(c(-0.5,0.3,0,0), "cm")) +
    geom_pointrange(aes(ymin=ci_left, ymax=ci_right),
                  width=.2,                    # Width of the error bars
                  position=position_dodge(.9)) +
    coord_flip() +
    scale_x_reverse(labels=paste("Q",c(1:15), sep=""), breaks=c(1:15)) + guides(fill = guide_legend(reverse = TRUE)) + # to get the right order of things
    scale_y_continuous(limits=c(0,10), breaks=c(1:9), expand = c(0, 0)) + ylab("response time [s]") +
    theme(panel.grid.major=element_line(color="gray90"), panel.grid.major.y=element_blank(), panel.grid.minor.y=element_blank())
  
  return (g_plot)
}
plotRobustAnova(plotData)

pdf(paste(GENERATED_IMAGES_DIR, "confidence-intervals.pdf", sep=''), width=5.5, height=5.5)
print(plotRobustAnova(plotData))
dev.off()

########################################################################################################################
# Multiplot function
# Adapted from: http://www.cookbook-r.com/Graphs/Multiple_graphs_on_one_page_%28ggplot2%29/
########################################################################################################################
# Multiple plot function
#
# ggplot objects can be passed in plotlist (as a list of ggplot objects)
# - cols:   Number of columns in layout
# - layout: A matrix specifying the layout. If present, 'cols' is ignored.
#
# If the layout is something like matrix(c(1,2,3,3), nrow=2, byrow=TRUE),
# then plot 1 will go in the upper left, 2 will go in the upper right, and
# 3 will go all the way across the bottom.
#
multiplot <- function(plots, cols=1, layout=NULL) {
  library(grid)
  
  numPlots = length(plots)
  
  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                     ncol = cols, nrow = ceiling(numPlots/cols))
  }
  
  if (numPlots==1) {
    print(plots[[1]])
    
  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))
    
    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))
      
      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}

########################################################################################################################
# Normality test - Shapiro-Wilk
########################################################################################################################
timeNormalitySummary = summaryBy(Time~Condition + Question, data=allData, 	FUN = function(x) { stat.desc(x, basic=FALSE, norm=TRUE) } )
correctnessNormalitySummary = summaryBy(Correct~Condition + Question, data=allData, 	FUN = function(x) { stat.desc(x, basic=FALSE, norm=TRUE) } )

########################################################################################################################
# Normality plots
########################################################################################################################
histogramAndFitPlot = function (data, xlabel='data') {
  df = data.frame(data)
  thePlot = ggplot(df, aes(data) ) +
  geom_histogram(aes(y = ..density..), colour = "black", fill = "white") +
  labs( y="Density", x=xlabel) +
  stat_function(fun =dnorm, args=list(mean=mean(data, na.rm = TRUE), sd = sd(data, na.rm = TRUE)), colour = "black", size=1)
  
  return (thePlot)
}

allHistogramsAndFitsTime = function(data) {
  res = with(data, by(data, c(data['Question'], data['Condition']),  function(x) histogramAndFitPlot(x$Time, paste(x$Question[[1]],x$Condition[[1]],sep='\n'))))
  multiplot(plots=res,cols=3)
}

allHistogramsAndFitsCorrectness = function(data) {
  res = with(data, by(data, c(data['Question'], data['Condition']),  function(x) histogramAndFitPlot(x$Correct, paste(x$Question[[1]],x$Condition[[1]],sep='\n'))))
  multiplot(plots=res,cols=3)
}

# Uncomment these to generate the normality plots. They take a while.
# pdf(paste(GENERATED_IMAGES_DIR, "normality-time.pdf", sep=''), width=10, height=40)
# print( allHistogramsAndFitsTime(allData) )
# dev.off()
# 
# pdf(paste(GENERATED_IMAGES_DIR, "normality-correctness.pdf", sep=''), width=10, height=40)
# print( allHistogramsAndFitsCorrectness(allData) )
# dev.off()